// This file is generated by kconfig_compiler_kf6 from incidenceeditor.kcfg.
// All changes you do to this file will be lost.
#ifndef INCIDENCEEDITORNG_INCIDENCEEDITORSETTINGSBASE_H
#define INCIDENCEEDITORNG_INCIDENCEEDITORSETTINGSBASE_H

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace IncidenceEditorNG {

class IncidenceEditorSettingsBase : public KConfigSkeleton
{
  public:
    enum EnumDefaultEmailAttachMethod { Ask, Link, InlineFull, InlineBody };

    IncidenceEditorSettingsBase( );
    ~IncidenceEditorSettingsBase() override;

    /**
      Set Default Email Attachment Method
    */
    void setDefaultEmailAttachMethod( int v )
    {
      if (!isDefaultEmailAttachMethodImmutable())
        mDefaultEmailAttachMethod = v;
    }

    /**
      Get Default Email Attachment Method
    */
    int defaultEmailAttachMethod() const
    {
      return mDefaultEmailAttachMethod;
    }

    /**
      Is Default Email Attachment Method Immutable
    */
    bool isDefaultEmailAttachMethodImmutable() const
    {
      return isImmutable( QStringLiteral( "DefaultEmailAttachMethod" ) );
    }

    /**
      Get Item object corresponding to DefaultEmailAttachMethod()
    */
    ItemEnum *defaultEmailAttachMethodItem()
    {
      return mDefaultEmailAttachMethodItem;
    }

  protected:
  public:

    // AttachmentHandling
    int mDefaultEmailAttachMethod;

  private:
    ItemEnum *mDefaultEmailAttachMethodItem;
};

}

#endif

